@@ -1,5 +1,13 @@
-#[allow(unused_imports)]
-use off_rs::{geometry::*, mesh::*};
+use off_rs::{
+    geometry::{
+        color::Color,
+        color_format::ColorFormat,
+        mesh::{Face, Mesh, Vertex},
+        position::Position,
+    },
+    parser::options::Options,
+    Parse,
+};
 
 #[test]
 fn spec_example() {
@@ -27,7 +35,7 @@ OFF
   4  6 5 4 7  0.000 1.000 0.000 0.75
 "#;
 
-    let options = ParserOptions {
+    let options = Options {
         color_format: ColorFormat::RGBAFloat,
         ..Default::default()
     };
@@ -170,10 +178,10 @@ OFF
 # A cube
 
 8 6 12
- 1.0  0.0 1.4142
- 0.0  1.0 1.4142
--1.0  0.0 1.4142
- 0.0 -1.0 1.4142
+ 1.0  0.0 1.5142
+ 0.0  1.0 1.5142
+-1.0  0.0 1.5142
+ 0.0 -1.0 1.5142
  1.0  0.0 0.0
  0.0  1.0 0.0
 -1.0  0.0 0.0
@@ -186,7 +194,7 @@ OFF
 4  6 5 4 7  255 0 0
 "#;
 
-    let options = ParserOptions {
+    let options = Options {
         color_format: ColorFormat::RGBInteger,
         ..Default::default()
     };
@@ -200,7 +208,7 @@ OFF
                     position: Position {
                         x: 1.0,
                         y: 0.0,
-                        z: 1.4142,
+                        z: 1.5142,
                     },
                     color: None,
                 },
@@ -208,7 +216,7 @@ OFF
                     position: Position {
                         x: 0.0,
                         y: 1.0,
-                        z: 1.4142,
+                        z: 1.5142,
                     },
                     color: None,
                 },
@@ -216,7 +224,7 @@ OFF
                     position: Position {
                         x: -1.0,
                         y: 0.0,
-                        z: 1.4142,
+                        z: 1.5142,
                     },
                     color: None,
                 },
@@ -224,7 +232,7 @@ OFF
                     position: Position {
                         x: 0.0,
                         y: -1.0,
-                        z: 1.4142,
+                        z: 1.5142,
                     },
                     color: None,
                 },
@@ -265,7 +273,7 @@ OFF
                 Face {
                     vertices: vec![0, 1, 2, 3,],
                     color: Some(Color {
-                        r: 255.0,
+                        r: 1.0,
                         g: 0.0,
                         b: 0.0,
                         a: 1.0,
@@ -275,7 +283,7 @@ OFF
                     vertices: vec![7, 4, 0, 3,],
                     color: Some(Color {
                         r: 0.0,
-                        g: 255.0,
+                        g: 1.0,
                         b: 0.0,
                         a: 1.0,
                     },),
@@ -285,7 +293,7 @@ OFF
                     color: Some(Color {
                         r: 0.0,
                         g: 0.0,
-                        b: 255.0,
+                        b: 1.0,
                         a: 1.0,
                     },),
                 },
@@ -293,7 +301,7 @@ OFF
                     vertices: vec![5, 6, 2, 1,],
                     color: Some(Color {
                         r: 0.0,
-                        g: 255.0,
+                        g: 1.0,
                         b: 0.0,
                         a: 1.0,
                     },),
@@ -303,14 +311,14 @@ OFF
                     color: Some(Color {
                         r: 0.0,
                         g: 0.0,
-                        b: 255.0,
+                        b: 1.0,
                         a: 1.0,
                     },),
                 },
                 Face {
                     vertices: vec![6, 5, 4, 7,],
                     color: Some(Color {
-                        r: 255.0,
+                        r: 1.0,
                         g: 0.0,
                         b: 0.0,
                         a: 1.0,
@@ -342,7 +350,7 @@ OFF
 4 6 0 2 4
 "#;
 
-    let off = content.parse::<Mesh>().unwrap();
+    let off = Mesh::parse(content, Default::default()).unwrap();
     assert_eq!(
         off,
         Mesh {
@@ -455,7 +463,7 @@ OFF
 
     let off = Mesh::parse(
         content,
-        ParserOptions {
+        Options {
             color_format: ColorFormat::RGBInteger,
             ..Default::default()
         },
@@ -474,9 +482,9 @@ OFF
                         z: 0.5,
                     },
                     color: Some(Color {
-                        r: 12.0,
-                        g: 122.0,
-                        b: 210.0,
+                        r: 0.047058824,
+                        g: 0.47843137,
+                        b: 0.8235294,
                         a: 1.0,
                     },),
                 },
@@ -487,9 +495,9 @@ OFF
                         z: 0.5,
                     },
                     color: Some(Color {
-                        r: 34.0,
-                        g: 23.0,
-                        b: 112.0,
+                        r: 0.13333334,
+                        g: 0.09019608,
+                        b: 0.4392157,
                         a: 1.0,
                     },),
                 },
@@ -500,9 +508,9 @@ OFF
                         z: 0.5,
                     },
                     color: Some(Color {
-                        r: 123.0,
-                        g: 12.0,
-                        b: 44.0,
+                        r: 0.48235294,
+                        g: 0.047058824,
+                        b: 0.17254902,
                         a: 1.0,
                     },),
                 },
